from setuptools import setup


setup(
    name='sqlLibraryNabeel',
    version='0.0.1',
    description = 'Process json and convert it into dataframes',
    py_modules=["sheets","sqlConverter"],
    package_dir={'':'src'},
    install_requires=[
        'autopep8~=1.5.4',
        'cachetools~=4.1.1',
        'certifi~=2020.6.20',
        'chardet~=3.0.4',
        'click~=7.1.2',
        'colorama~=0.4.4',
        'Flask~=1.1.2',
        'google-api-core~=1.23.0',
        'google-api-python-client~=1.12.5',
        'google-auth~=1.23.0',
        'google-auth-httplib2~=0.0.4',
        'google-auth-oauthlib~=0.4.2',
        'googleapis-common-protos~=1.52.0',
        'gspread~=3.6.0',
        'httplib2~=0.18.1',
        'idna~=2.10',
        'isort~=5.6.4',
        'itsdangerous~=1.1.0',
        'Jinja2~=2.11.2',
        'lazy-object-proxy~=1.4.3',
        'MarkupSafe~=1.1.1',
        'mccabe~=0.6.1',
        'numpy~=1.19.3',
        'oauth2client~=4.1.3',
        'oauthlib~=3.1.0',
        'pandas~=1.1.4',
        'pandasql~=0.7.3',
        'protobuf~=3.13.0',
        'pyasn1~=0.4.8',
        'pyasn1-modules~=0.2.8',
        'pycodestyle~=2.6.0',
        'pylint~=2.6.0',
        'python-dateutil~=2.8.1',
        'pytz~=2020.1',
        'requests~=2.24.0',
        'requests-oauthlib~=1.3.0',
        'rsa~=4.6',
        'Sheets~=0.6',
        'six~=1.15.0',
        'SQLAlchemy~=1.3.20',
        'toml~=0.10.2',
        'Unidecode~=1.1.1',
        'uritemplate~=3.0.1',
        'urllib3~=1.25.11',
        'Werkzeug~=1.0.1',
        'wrapt~=1.12.1'
    ],
    url='https://github.com/#',
    author='Rusab Naeem',
    author_email='rusab.khan@lyftron.com'

)


from sheets import filterData,deleteData,updateData
import re

slct = 'select'
updt = 'update'
dlte = 'delete'


def queryMaker(query):
    commands = query.split()
    col = []
    values = []

    if (commands[0].lower() == slct.lower()):
        columnsPart = query.split('where ')[1]
        columns = columnsPart.rsplit(' or ')
        displayValues = query.split(',')
        table = query.split('from ', maxsplit=1)[1]
        table = table.split()

        displayValues[0] = re.sub('select ', '', displayValues[0], flags=re.IGNORECASE)
        displayValues[len(displayValues) - 1] = displayValues[len(displayValues) - 1].split(' from')[0]

        for c in columns:
            col.append(c.split('=')[0])
            values.append(c.split('=')[1])
        if ("*" in commands[1]):
            # print()
            displayValues = '*'
            filterData(table[0], col, values, displayValues)
        else:
            print()
            filterData(table[0], col, values, displayValues)

    elif (commands[0].lower() == updt.lower()):
        table = query.split(' SET', maxsplit=1)[0]
        table = re.sub('update ', '', table, flags=re.IGNORECASE)
        columnsPart = query.split('WHERE ')[1]
        columns = columnsPart.rsplit(' or ')
        updValues = query.split(',')
        updValues[0] = (re.sub('Update '+table+' set ', '', updValues[0], flags=re.IGNORECASE)).split('=')[1]
        updValues[len(updValues) - 1] = (updValues[len(updValues) - 1].split(' WHERE')[0]).split('=')[1]

        for c in columns:
            col.append(c.split('=')[0])
            values.append(c.split('=')[1])
        updateData(table, col, values,updValues)

    elif (commands[0].lower() == dlte.lower()):
        columnsPart = query.split('where ')[1]
        columns = columnsPart.rsplit(' or ')
        table = query.split('from ', maxsplit=1)[1]
        table = table.split()
        for c in columns:
            col.append(c.split('=')[0])
            values.append(c.split('=')[1])
        deleteData(table[0], col, values)

        # print( sheets.deleteData(colClause[1]))


#queryMaker('UPDATE S1 SET Keyword=bookkeeping WHERE Keyword=Keyword')
queryMaker('select * from S1 where Keyword=High')

# lst = [("select keyword"),("bb8"),("ccc8"),("dddddd8")]

# print("rusab from spreadsheet where keyword=bookkeeping".split(' from', 1)[0])
# print([s.strip('select ') for s in lst]) # remove the 8 from the string borders
# print([s.replace('8', '') for s in lst]) # remove all the 8s



import gspread
from oauth2client.service_account import ServiceAccountCredentials
from dateutil.parser import parse
import pandas as pd

data = []
gs = None




def initialize(spreadsheetName):
    scope = ["https://spreadsheets.google.com/feeds", 'https://www.googleapis.com/auth/spreadsheets',
             "https://www.googleapis.com/auth/drive.file", "https://www.googleapis.com/auth/drive"]
    Creds = ServiceAccountCredentials.from_json_keyfile_name(
        "creds.json", scope)
    client = gspread.authorize(Creds)
    return client.open(spreadsheetName)


gs = initialize("Lyftron")  # activate all sheet methods from here
filtered_dict = []


def makeSchema():
    sheets = gs.worksheets()
    schema = []
    for s in sheets:
        headers = (s.row_values(1))
        # read first 100 values to determine datatypes
        res = (gs.values_get(s.title + "!A2:AZ100"))
        table = pd.DataFrame(res['values'])
        schema.append(checkDtype(table, headers))
    return schema


def makeLookup():
    lookUp = dt
    sheets = []
    for d in lookUp:
        for s in d:
            d[d.index(s)] = s.split(":", maxsplit=1)[0]
    for s in gs.worksheets():
        sheets.append(s.title)
    lookUp.append(sheets)
    return lookUp


def checkDtype(table, headers):
    digit = 0
    date = 0
    inc = 0
    for c in table:
        for v in table[c]:
            if v.isdigit() | v.replace('.', '').replace(',', '').isdigit():
                digit = digit + 1
            elif is_date(v):
                date = date + 1
        if digit == len(table[c]):
            if table[c].str.len().max() > 5:
                headers[inc] = headers[inc] + ":nvarchar"
                inc = inc + 1
                digit = 0
            else:
                headers[inc] = headers[inc] + ":int"
                inc = inc + 1
                digit = 0
        elif date == len(table[c]):
            headers[inc] = headers[inc] + ":date"
            inc = inc + 1
            date = 0
        else:
            if table[c].str.len().max() > 5:
                headers[inc] = headers[inc] + ":nvarchar"
                inc = inc + 1
            else:
                headers[inc] = headers[inc] + ":varchar"
                inc = inc + 1
    return headers


def is_date(string, fuzzy=False):
    """
    Return whether the string can be interpreted as a date.
    :param string: str, string to check for date
    :param fuzzy: bool, ignore unknown tokens in string if True
    """
    try:
        parse(string, fuzzy=fuzzy)
        return True

    except ValueError:
        return False


def makeTable():
    sheets = gs.worksheets()
    table = None
    for s in sheets:
        # read first 100 values to determine datatypes
        res = (gs.values_get(s.title + "!A1:AZ100"))
        t1 = pd.DataFrame(res['values'])
        table = pd.concat([table, t1], axis=1)
    print(table)



dt = makeSchema()
print(dt)

lookUp = makeLookup()


def filterData(table, columns, value, displayValues):
    # if len(columns)>1 & len(value)>1:
    results = []
    arrIndex = (lookUp[len(lookUp) - 1].index(table))

    for col in columns:
        colIndex = lookUp[arrIndex].index(col) + 1
        c = gs.worksheet(table).findall(
            value[columns.index(col)], None, colIndex)
        for s in c:
            results.append(gs.worksheet(table).row_values(s.row))

    if displayValues == '*':
        for res in results:
            print(res)
            print(pd.DataFrame(res))
    else:
        for res in results:
            for col in displayValues:
                colIndex = lookUp[arrIndex].index(col)
                print(res[colIndex])


def updateData(table, columns, value, updValues):
    arrIndex = (lookUp[len(lookUp) - 1].index(table))
    matches = []

    for col in columns:
        colIndex = lookUp[arrIndex].index(col) + 1
        matches = gs.worksheet(table).findall(
            value[columns.index(col)], None, colIndex)
    for i, value in enumerate(updValues):
        matches[i].value = value
    gs.worksheet(table).update_cells(matches)


def deleteData(table, columns, value):
    arrIndex = (lookUp[len(lookUp) - 1].index(table))
    matches = []

    for col in columns:
        colIndex = lookUp[arrIndex].index(col) + 1
        matches = gs.worksheet(table).findall(
            value[columns.index(col)], None, colIndex)
    for match in reversed(matches):
        gs.worksheet(table).delete_row(match.row)
    # return gs.delete_row(delIndex)
